import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import java.util.ArrayList;

import org.lekan.graphics.GraphicsProgram;
import org.lekan.graphics.SGEllipse;
import org.lekan.graphics.SGLine;
import org.lekan.graphics.SGObject;
import org.lekan.graphics.SGQuadCurve;
import org.lekan.graphics.SGRect;
import org.lekan.graphics.SGRoundedRect;
import org.lekan.graphics.SGText;


/**
 * Here's an intro/review to using Java, and an intro to using the SGGraphics package.
 * 
 * Your task: build a simple drawing program that does, at the minimum, the following:
 *  (1) Buttons that lets you choose whether to draw an ellipse, a rectangle, a line, a rounded line, or a quadratic curve
 *  (2) When the program first starts, the main canvas should display a nice welcoming message and/or graphics.
 *  (3) As soon as the user starts drawing, this welcome message should disappear
 *  (4) To draw, click and drag the mouse. The shape should display as you are dragging.
 *  (5) When released, the shape will stay on the screen.
 *  (6) A clear button will clear everything on the screen.
 * 
 * @author Martin Wickham
 * @version 1.0__6/21/2011
 *
 */
public class EPGYDraw extends GraphicsProgram {

//States - for determining what shape to draw
private final short NULL_STATE = 0;
private final short ELLIPSE_STATE = 1;
private final short RECT_STATE = 2;
private final short LINE_STATE = 3;
private final short RND_RECT_STATE = 4;
private final short QUAD_STATE = 5;

//Buttons - for determining which button was pressed
private final String ELLIPSE_BTN_NAME = "Ellipse";
private final String CLEAR_BTN_NAME = "Clear All";
private final String LINE_BTN_NAME = "Line";
private final String RND_RECT_BTN_NAME = "Rounded Rectangle";
private final String RECT_BTN_NAME = "Rectangle";
private final String QUAD_BTN_NAME = "Quadratic Curve";
private final String UNDO_BTN_NAME = "Undo";

//Quadratic Variables - for drawing the quadratic line
private int startLineX;
private int startLineY;
private int endLineX;
private int endLineY;
private boolean canSetThird;

//Other Variables - used in drawing everything
private int startDragX;
private int startDragY;
private short state;
private ArrayList<SGObject> stuff;

	/**
	 * The constructor for objects of class EPGYDraw
	 */
	public EPGYDraw() {
		canSetThird = false;
		state = NULL_STATE;
		stuff = new ArrayList<SGObject>();
	}

	/**
	 * Starts the program
	 * @param args required by java
	 */
	public static void main(String[] args) {
		new EPGYDraw();
	}
	
	/**
	 * Creates the buttons, the title, and the opening message
	 */
	@Override
	public void setup() {
		this.getFrame().setTitle("EPGYDraw!");
		addButton(ELLIPSE_BTN_NAME, SOUTH);
		addButton(RECT_BTN_NAME, SOUTH);
		addButton(LINE_BTN_NAME, SOUTH);
		addButton(RND_RECT_BTN_NAME, SOUTH);
		addButton(QUAD_BTN_NAME, SOUTH);
		addButton(UNDO_BTN_NAME, SOUTH);
		addButton(CLEAR_BTN_NAME, SOUTH);
		this.getFrame().addObject(new SGText("Welcome to the program!  Click a button to start drawing.", 200, 300));
	}
	
	/**
	 * This method sets a shape if a shape button is pressed, or erases if an erase button is pressed
	 */
	public void actionPerformed(ActionEvent e) {
		if (canSetThird) {
			stuff.remove(stuff.size()-1);
		}
		if (e.getActionCommand().equals(ELLIPSE_BTN_NAME)){
			state = ELLIPSE_STATE;
		}
		else if (e.getActionCommand().equals(RECT_BTN_NAME)){
			state = RECT_STATE;
		}
		else if (e.getActionCommand().equals(LINE_BTN_NAME)){
			state = LINE_STATE;
		}
		else if (e.getActionCommand().equals(RND_RECT_BTN_NAME)){
			state = RND_RECT_STATE;
		}
		else if (e.getActionCommand().equals(QUAD_BTN_NAME)){
			state = QUAD_STATE;
		}
		else if (e.getActionCommand().equals(CLEAR_BTN_NAME)){
			this.getFrame().clearGraphics();
			stuff.removeAll(stuff);
		}
		else if (e.getActionCommand().equals(UNDO_BTN_NAME)){
			this.getFrame().clearGraphics();
			if (!canSetThird && stuff.size() > 0) {
				stuff.remove(stuff.size()-1);
			}
			drawAllStuff();
		}
		canSetThird = false;
		
	}
	
	/**
	 * This method sets the beginning point of a drag
	 */
	public void mousePressed(MouseEvent e) {
		startDragX = e.getX();
		startDragY = e.getY();
		if (canSetThird) {
			stuff.remove(stuff.size()-1);
		}
	}
	
	/**
	 * This method draws a shape based on the user's choice of shape and the mouse position
	 */
	public void mouseDragged(MouseEvent e) {
		int xPos = e.getX();
		int yPos = e.getY();
		this.getFrame().clearGraphics();
		drawAllStuff();
		switch (state) {
		case ELLIPSE_STATE:
			this.getFrame().drawEllipse(startDragX, startDragY, Math.abs(startDragX - xPos), Math.abs(startDragY - yPos));
			break;
		case RECT_STATE:
			this.getFrame().drawRect(Math.min(startDragX, xPos), Math.min(startDragY, yPos), Math.abs(startDragX - xPos), Math.abs(startDragY - yPos));
			break;
		case LINE_STATE:
			this.getFrame().drawLine(startDragX, startDragY, xPos, yPos);
			break;
		case RND_RECT_STATE:
			this.getFrame().drawRoundedRect(Math.min(startDragX, xPos), Math.min(startDragY, yPos), Math.abs(startDragX - xPos), Math.abs(startDragY - yPos));
			break;
		case QUAD_STATE:
			if (!canSetThird) {
				this.getFrame().drawLine(startDragX, startDragY, xPos, yPos);
			}
			else {
			this.getFrame().drawQuadCurve(startLineX, startLineY, xPos, yPos, endLineX, endLineY);
			}
			break;
		}
		
	}
	
	/**
	 * This method finalizes the shapes drawn in mouseDragged(MouseEvent e)
	 */
	public void mouseReleased(MouseEvent e) {
		int xPos = e.getX();
		int yPos = e.getY();
		switch (state) {
		case ELLIPSE_STATE:
			stuff.add(new SGEllipse(startDragX, startDragY, Math.abs(startDragX - xPos), Math.abs(startDragY - yPos)));
			break;
		case RECT_STATE:
			stuff.add(new SGRect(Math.min(startDragX, xPos), Math.min(startDragY, yPos), Math.abs(startDragX - xPos), Math.abs(startDragY - yPos)));
			break;
		case LINE_STATE:
			stuff.add(new SGLine(startDragX, startDragY, xPos, yPos));
			break;
		case RND_RECT_STATE:
			stuff.add(new SGRoundedRect(Math.min(startDragX, xPos), Math.min(startDragY, yPos), Math.abs(startDragX - xPos), Math.abs(startDragY - yPos)));
			break;
		case QUAD_STATE:
			if (!canSetThird) {
				stuff.add(new SGLine(startDragX, startDragY, xPos, yPos));
				startLineX = startDragX;
				startLineY = startDragY;
				endLineX = xPos;
				endLineY = yPos;
				canSetThird = true;
			}
			else {
			stuff.add(new SGQuadCurve(startLineX, startLineY, xPos, yPos, endLineX, endLineY));
			canSetThird = false;
			}
			break;
		}
	}
	
	/**
	 * This method does nothing. (yet)
	 */
	@Override
	public void run() {
		// TODO: The main run of the program!
	}
	
	/**
	 * Draws all shapes stored in stuff
	 */
	public void drawAllStuff() {
		for (int c = 0; c < stuff.size(); c++){
			this.getFrame().addObject(stuff.get(c));
		}
	}	
}
